@@ -14,9 +14,7 @@ path_hack()
 class TestsFailedError(Exception):
     pass
 
-print('Testing optional dependencies')
 
-import sympy
 test_list = [
     # numpy
     '*numpy*',
@@ -24,18 +22,28 @@ test_list = [
     'sympy/matrices/',
     'sympy/physics/quantum/',
     'sympy/utilities/tests/test_lambdify.py',
+    'sympy/physics/control/',
 
     # scipy
     '*scipy*',
 
+    # matplotlib
+    'sympy/plotting/',
+
     # llvmlite
     '*llvm*',
 
     # aesara
     '*aesara*',
 
+    # jax
+    '*jax*',
+
     # gmpy
-    'polys',
+    'sympy/polys',
+
+    # gmpy, numpy, scipy, autowrap, matplotlib
+    'sympy/external',
 
     # autowrap
     '*autowrap*',
@@ -46,13 +54,11 @@ test_list = [
     # antlr, lfortran, clang
     'sympy/parsing/',
 
-    # matchpy
-    '*rubi*',
-
     # codegen
     'sympy/codegen/',
     'sympy/utilities/tests/test_codegen',
     'sympy/utilities/_compilation/tests/test_compilation',
+    'sympy/external/tests/test_codegen.py',
 
     # cloudpickle
     'pickling',
@@ -61,15 +67,19 @@ test_list = [
     'sympy/logic',
     'sympy/assumptions',
 
-    #stats
+    # stats
     'sympy/stats',
 
+    # lxml
+    "sympy/utilities/tests/test_mathml.py",
 ]
 
+
 blacklist = [
     'sympy/physics/quantum/tests/test_circuitplot.py',
 ]
 
+
 doctest_list = [
     # numpy
     'sympy/matrices/',
@@ -78,6 +88,9 @@ doctest_list = [
     # scipy
     '*scipy*',
 
+    # matplotlib
+    'sympy/plotting/',
+
     # llvmlite
     '*llvm*',
 
@@ -85,7 +98,7 @@ doctest_list = [
     '*aesara*',
 
     # gmpy
-    'polys',
+    'sympy/polys',
 
     # autowrap
     '*autowrap*',
@@ -96,9 +109,6 @@ doctest_list = [
     # antlr, lfortran, clang
     'sympy/parsing/',
 
-    # matchpy
-    '*rubi*',
-
     # codegen
     'sympy/codegen/',
 
@@ -109,21 +119,24 @@ doctest_list = [
     #stats
     'sympy/stats',
 
+    # lxml
+    "sympy/utilities/mathml/",
 ]
 
-if not (sympy.test(*test_list, verbose=True, blacklist=blacklist) and sympy.doctest(*doctest_list)):
-    raise TestsFailedError('Tests failed')
+
+print('Testing optional dependencies')
 
 
-print('Testing MATPLOTLIB')
-# Set matplotlib so that it works correctly in headless Travis. We have to do
-# this here because it doesn't work after the sympy plotting module is
-# imported.
-import matplotlib
-matplotlib.use("Agg")
-import sympy
-# Unfortunately, we have to use subprocess=False so that the above will be
-# applied, so no hash randomization here.
-if not (sympy.test('sympy/plotting', 'sympy/physics/quantum/tests/test_circuitplot.py',
-    subprocess=False) and sympy.doctest('sympy/plotting', subprocess=False)):
-    raise TestsFailedError('Tests failed')
+from sympy import test, doctest
+
+
+tests_passed = test(*test_list, blacklist=blacklist, force_colors=True)
+doctests_passed = doctest(*doctest_list, force_colors=True)
+
+
+if not tests_passed and not doctests_passed:
+    raise TestsFailedError('Tests and doctests failed')
+elif not tests_passed:
+    raise TestsFailedError('Doctests passed but tests failed')
+elif not doctests_passed:
+    raise TestsFailedError('Tests passed but doctests failed')
