@@ -38,13 +38,7 @@ def build_html(docsdir, outputdir, version):
 
 def build_latex(docsdir, outputdir, version):
     run('make', 'clean', cwd=docsdir)
-    run('make', 'latex', cwd=docsdir)
-
-    latexdir = join(docsdir, '_build', 'latex')
-    env = os.environ.copy()
-    env['LATEXMKOPTS'] = '-xelatex -silent'
-    run('make', 'clean', cwd=latexdir, env=env)
-    run('make', 'all', cwd=latexdir, env=env)
+    run('make', 'latexpdf', cwd=docsdir)
 
     srcfilename = 'sympy-%s.pdf' % (version,)
     dstfilename = 'sympy-docs-pdf-%s.pdf' % (version,)
