@@ -32,13 +32,14 @@ git_whitelist = {
     '.gitattributes',
     '.gitignore',
     '.mailmap',
-    # Travis and CI
-    '.travis.yml',
+    # CI
+    '.github/PULL_REQUEST_TEMPLATE.md',
     '.github/workflows/runtests.yml',
     '.github/workflows/ci-sage.yml',
     '.github/workflows/comment-on-pr.yml',
     '.github/workflows/release.yml',
     '.github/workflows/docs-preview.yml',
+    '.github/workflows/checkconflict.yml',
     '.ci/durations.json',
     '.ci/generate_durations_log.sh',
     '.ci/parse_durations_log.py',
@@ -50,16 +51,12 @@ git_whitelist = {
     '.coveragerc',
     'CODEOWNERS',
     'asv.conf.actions.json',
-    'asv.conf.travis.json',
-    'coveragerc_travis',
     'codecov.yml',
     'pytest.ini',
     'MANIFEST.in',
     'banner.svg',
     # Code of conduct
     'CODE_OF_CONDUCT.md',
-    # Pull request template
-    'PULL_REQUEST_TEMPLATE.md',
     # Contributing guide
     'CONTRIBUTING.md',
     # Nothing from bin/ should be shipped unless we intend to install it. Most
@@ -92,14 +89,12 @@ git_whitelist = {
     'bin/test_py2_import.py',
     'bin/test_setup.py',
     'bin/test_submodule_imports.py',
-    'bin/test_travis.sh',
     'bin/test_optional_dependencies.py',
     'bin/test_sphinx.sh',
     'bin/mailmap_check.py',
     'bin/test_symengine.py',
     'bin/test_tensorflow.py',
     'bin/test_pyodide.mjs',
-
     # The notebooks are not ready for shipping yet. They need to be cleaned
     # up, and preferably doctested.  See also
     # https://github.com/sympy/sympy/issues/6039.
@@ -126,7 +121,6 @@ git_whitelist = {
     # This stuff :)
     'release/.gitignore',
     'release/README.md',
-    'release/Vagrantfile',
     'release/compare_tar_against_git.py',
     'release/update_docs.py',
     'release/build_docs.py',
@@ -136,13 +130,8 @@ git_whitelist = {
     'release/sha256.py',
     'release/authors.py',
     'release/ci_release_script.sh',
-    # This is just a distribute version of setup.py. Used mainly for setup.py
-    # develop, which we don't care about in the release tarball
-    'setupegg.py',
     # pytest stuff
     'conftest.py',
-    # Encrypted deploy key for deploying dev docs to GitHub
-    'github_deploy_key.enc',
     }
 
 # Files that should be in the tarball should not be in git
@@ -196,7 +185,7 @@ def compare_tar_against_git(tarname, gitroot):
 
     See the bottom of the file for the whitelists.
     """
-    git_lsfiles = set(i.strip() for i in run('git', 'ls-files', cwd=gitroot))
+    git_lsfiles = {i.strip() for i in run('git', 'ls-files', cwd=gitroot)}
     tar_output_orig = set(run('tar', 'tf', tarname))
     tar_output = set()
     for file in tar_output_orig:
