@@ -1,14 +1,45 @@
-from sympy import (Add, Basic, Expr, S, Symbol, Wild, Float, Integer, Rational, I,
-                   sin, cos, tan, exp, log, nan, oo, sqrt, symbols, Integral, sympify,
-                   WildFunction, Poly, Function, Derivative, Number, pi, NumberSymbol, zoo,
-                   Piecewise, Mul, Pow, nsimplify, ratsimp, trigsimp, radsimp, powsimp,
-                   simplify, together, collect, factorial, apart, combsimp, factor, refine,
-                   cancel, Tuple, default_sort_key, DiracDelta, gamma, Dummy, Sum, E,
-                   exp_polar, expand, diff, O, Heaviside, Si, Max, UnevaluatedExpr,
-                   integrate, gammasimp, Gt)
-from sympy.core.expr import ExprBuilder, unchanged
-from sympy.core.function import AppliedUndef
+from sympy.assumptions.refine import refine
+from sympy.concrete.summations import Sum
+from sympy.core.add import Add
+from sympy.core.basic import Basic
+from sympy.core.containers import Tuple
+from sympy.core.expr import (ExprBuilder, unchanged, Expr,
+    UnevaluatedExpr)
+from sympy.core.function import (Function, expand, WildFunction,
+    AppliedUndef, Derivative, diff, Subs)
+from sympy.core.mul import Mul
+from sympy.core.numbers import (NumberSymbol, E, zoo, oo, Float, I,
+    Rational, nan, Integer, Number, pi, _illegal)
+from sympy.core.power import Pow
+from sympy.core.relational import Ge, Lt, Gt, Le
+from sympy.core.singleton import S
+from sympy.core.sorting import default_sort_key
+from sympy.core.symbol import Symbol, symbols, Dummy, Wild
+from sympy.core.sympify import sympify
+from sympy.functions.combinatorial.factorials import factorial
+from sympy.functions.elementary.exponential import exp_polar, exp, log
+from sympy.functions.elementary.miscellaneous import sqrt, Max
+from sympy.functions.elementary.piecewise import Piecewise
+from sympy.functions.elementary.trigonometric import tan, sin, cos
+from sympy.functions.special.delta_functions import (Heaviside,
+    DiracDelta)
+from sympy.functions.special.error_functions import Si
+from sympy.functions.special.gamma_functions import gamma
+from sympy.integrals.integrals import integrate, Integral
 from sympy.physics.secondquant import FockState
+from sympy.polys.partfrac import apart
+from sympy.polys.polytools import factor, cancel, Poly
+from sympy.polys.rationaltools import together
+from sympy.series.order import O
+from sympy.sets.sets import FiniteSet
+from sympy.simplify.combsimp import combsimp
+from sympy.simplify.gammasimp import gammasimp
+from sympy.simplify.powsimp import powsimp
+from sympy.simplify.radsimp import collect, radsimp
+from sympy.simplify.ratsimp import ratsimp
+from sympy.simplify.simplify import simplify, nsimplify
+from sympy.simplify.trigsimp import trigsimp
+from sympy.tensor.indexed import Indexed
 from sympy.physics.units import meter
 
 from sympy.testing.pytest import raises, XFAIL
@@ -16,6 +47,9 @@ from sympy.testing.pytest import raises, XFAIL
 from sympy.abc import a, b, c, n, t, u, x, y, z
 
 
+f, g, h = symbols('f,g,h', cls=Function)
+
+
 class DummyNumber:
     """
     Minimal implementation of a number that works with SymPy.
@@ -102,7 +136,7 @@ class F1_1(DummyNumber):
 i5 = I5()
 f1_1 = F1_1()
 
-# basic sympy objects
+# basic SymPy objects
 basic_objs = [
     Rational(2),
     Float("1.3"),
@@ -231,10 +265,12 @@ class NonExpr(Basic, NonBasic):
     pass
 
 
-class SpecialOp(Basic):
+class SpecialOp():
     '''Represents the results of operations with NonBasic and NonExpr'''
     def __new__(cls, op, arg1, arg2):
-        return Basic.__new__(cls, op, arg1, arg2)
+        obj = object.__new__(cls)
+        obj.args = (op, arg1, arg2)
+        return obj
 
 
 class NonArithmetic(Basic):
@@ -334,7 +370,7 @@ def test_cooperative_operations():
 
 
 def test_relational():
-    from sympy import Lt
+    from sympy.core.relational import Lt
     assert (pi < 3) is S.false
     assert (pi <= 3) is S.false
     assert (pi > 3) is S.true
@@ -349,7 +385,6 @@ def test_relational():
 
 
 def test_relational_assumptions():
-    from sympy import Lt, Gt, Le, Ge
     m1 = Symbol("m1", nonnegative=False)
     m2 = Symbol("m2", positive=False)
     m3 = Symbol("m3", nonpositive=False)
@@ -442,15 +477,15 @@ def test_as_leading_term():
     raises(ValueError, lambda: (x + 1).as_leading_term(1))
 
     # https://github.com/sympy/sympy/issues/21177
-    f = -3*x + (x + Rational(3, 2) - sqrt(3)*S.ImaginaryUnit/2)**2\
+    e = -3*x + (x + Rational(3, 2) - sqrt(3)*S.ImaginaryUnit/2)**2\
         - Rational(3, 2) + 3*sqrt(3)*S.ImaginaryUnit/2
-    assert f.as_leading_term(x) == \
+    assert e.as_leading_term(x) == \
         (12*sqrt(3)*x - 12*S.ImaginaryUnit*x)/(4*sqrt(3) + 12*S.ImaginaryUnit)
 
     # https://github.com/sympy/sympy/issues/21245
-    f = 1 - x - x**2
-    fi = (1 + sqrt(5))/2
-    assert f.subs(x, y + 1/fi).as_leading_term(y) == \
+    e = 1 - x - x**2
+    d = (1 + sqrt(5))/2
+    assert e.subs(x, y + 1/d).as_leading_term(y) == \
         (-576*sqrt(5)*y - 1280*y)/(256*sqrt(5) + 576)
 
 
@@ -534,7 +569,6 @@ def test_atoms():
         {1 + x*(2 + y) + exp(3 + z), 2 + y, 3 + z}
 
     # issue 6132
-    f = Function('f')
     e = (f(x) + sin(x) + 2)
     assert e.atoms(AppliedUndef) == \
         {f(x)}
@@ -594,6 +628,8 @@ def test_is_polynomial():
     assert (
         (x**2)*(y**2) + x*(y**2) + y*x + exp(x)).is_polynomial(x, y) is False
 
+    assert (1/f(x) + 1).is_polynomial(f(x)) is False
+
 
 def test_is_rational_function():
     assert Integer(1).is_rational_function() is True
@@ -618,10 +654,11 @@ def test_is_rational_function():
     assert (sin(y)/x).is_rational_function(x) is True
     assert (sin(y)/x).is_rational_function(x, y) is False
 
-    assert (S.NaN).is_rational_function() is False
-    assert (S.Infinity).is_rational_function() is False
-    assert (S.NegativeInfinity).is_rational_function() is False
-    assert (S.ComplexInfinity).is_rational_function() is False
+    for i in _illegal:
+        assert not i.is_rational_function()
+        for d in (1, x):
+            assert not (i/d).is_rational_function()
+
 
 def test_is_meromorphic():
     f = a/x**2 + b + x + c*x**2
@@ -635,10 +672,10 @@ def test_is_meromorphic():
     assert g.is_meromorphic(x, zoo) is False
 
     n = Symbol('n', integer=True)
-    h = sin(1/x)**n*x
-    assert h.is_meromorphic(x, 0) is False
-    assert h.is_meromorphic(x, 1) is True
-    assert h.is_meromorphic(x, zoo) is False
+    e = sin(1/x)**n*x
+    assert e.is_meromorphic(x, 0) is False
+    assert e.is_meromorphic(x, 1) is True
+    assert e.is_meromorphic(x, zoo) is False
 
     e = log(x)**pi
     assert e.is_meromorphic(x, 0) is False
@@ -793,6 +830,10 @@ def test_as_numer_denom():
     assert ((A*B*C)**-1).as_numer_denom() == ((A*B*C)**-1, 1)
     assert ((A*B*C)**-1/x).as_numer_denom() == ((A*B*C)**-1, x)
 
+    # the following morphs from Add to Mul during processing
+    assert Add(0, (x + y)/z/-2, evaluate=False).as_numer_denom(
+        ) == (-x - y, 2*z)
+
 
 def test_trunc():
     import math
@@ -864,29 +905,28 @@ def test_as_independent():
     eq = Add(x, -x, 2, -3, evaluate=False)
     assert eq.as_independent(x) == (-1, Add(x, -x, evaluate=False))
     eq = Mul(x, 1/x, 2, -3, evaluate=False)
-    eq.as_independent(x) == (-6, Mul(x, 1/x, evaluate=False))
+    assert eq.as_independent(x) == (-6, Mul(x, 1/x, evaluate=False))
 
     assert (x*y).as_independent(z, as_Add=True) == (x*y, 0)
 
 @XFAIL
 def test_call_2():
     # TODO UndefinedFunction does not subclass Expr
-    f = Function('f')
     assert (2*f)(x) == 2*f(x)
 
 
 def test_replace():
-    f = log(sin(x)) + tan(sin(x**2))
+    e = log(sin(x)) + tan(sin(x**2))
 
-    assert f.replace(sin, cos) == log(cos(x)) + tan(cos(x**2))
-    assert f.replace(
+    assert e.replace(sin, cos) == log(cos(x)) + tan(cos(x**2))
+    assert e.replace(
         sin, lambda a: sin(2*a)) == log(sin(2*x)) + tan(sin(2*x**2))
 
     a = Wild('a')
     b = Wild('b')
 
-    assert f.replace(sin(a), cos(a)) == log(cos(x)) + tan(cos(x**2))
-    assert f.replace(
+    assert e.replace(sin(a), cos(a)) == log(cos(x)) + tan(cos(x**2))
+    assert e.replace(
         sin(a), lambda a: sin(2*a)) == log(sin(2*x)) + tan(sin(2*x**2))
     # test exact
     assert (2*x).replace(a*x + b, b - a, exact=True) == 2*x
@@ -929,7 +969,6 @@ def test_replace():
 
     # https://groups.google.com/forum/#!topic/sympy/8wCgeC95tz0
     n1, n2, n3 = symbols('n1:4', commutative=False)
-    f = Function('f')
     assert (n1*f(n2)).replace(f, lambda x: x) == n1*n2
     assert (n3*f(n2)).replace(f, lambda x: x) == n3*n2
 
@@ -985,15 +1024,12 @@ def test_count():
     assert expr.count(sin(a)) == 1
     assert expr.count(lambda u: type(u) is sin) == 1
 
-    f = Function('f')
     assert f(x).count(f(x)) == 1
     assert f(x).diff(x).count(f(x)) == 1
     assert f(x).diff(x).count(x) == 2
 
 
 def test_has_basics():
-    f = Function('f')
-    g = Function('g')
     p = Wild('p')
 
     assert sin(x).has(x)
@@ -1044,8 +1080,6 @@ def test_has_multiple():
 
 def test_has_piecewise():
     f = (x*y + 3/y)**(3 + 2)
-    g = Function('g')
-    h = Function('h')
     p = Piecewise((g(x), x < -1), (1, x <= 1), (f, True))
 
     assert p.has(x)
@@ -1097,20 +1131,19 @@ def test_has_integrals():
 
 
 def test_has_tuple():
-    f = Function('f')
-    g = Function('g')
-    h = Function('h')
-
     assert Tuple(x, y).has(x)
     assert not Tuple(x, y).has(z)
     assert Tuple(f(x), g(x)).has(x)
     assert not Tuple(f(x), g(x)).has(y)
     assert Tuple(f(x), g(x)).has(f)
     assert Tuple(f(x), g(x)).has(f(x))
-    assert not Tuple(f, g).has(x)
-    assert Tuple(f, g).has(f)
-    assert not Tuple(f, g).has(h)
-    assert Tuple(True).has(True) is True  # .has(1) will also be True
+    # XXX to be deprecated
+    #assert not Tuple(f, g).has(x)
+    #assert Tuple(f, g).has(f)
+    #assert not Tuple(f, g).has(h)
+    assert Tuple(True).has(True)
+    assert Tuple(True).has(S.true)
+    assert not Tuple(True).has(1)
 
 
 def test_has_units():
@@ -1144,6 +1177,10 @@ def test_as_poly_as_expr():
 
     assert p.as_poly() == p
 
+    # https://github.com/sympy/sympy/issues/20610
+    assert S(2).as_poly() is None
+    assert sqrt(2).as_poly(extension=True) is None
+
     raises(AttributeError, lambda: Tuple(x, x).as_poly(x))
     raises(AttributeError, lambda: Tuple(x ** 2, x, y).as_poly(x))
 
@@ -1244,12 +1281,16 @@ def test_as_coeff_exponent():
         (log(2)/(2 + pi), 0)
     # issue 4784
     D = Derivative
-    f = Function('f')
     fx = D(f(x), x)
     assert fx.as_coeff_exponent(f(x)) == (fx, 0)
 
 
 def test_extractions():
+    for base in (2, S.Exp1):
+        assert Pow(base**x, 3, evaluate=False
+            ).extract_multiplicatively(base**x) == base**(2*x)
+        assert (base**(5*x)).extract_multiplicatively(
+            base**(3*x)) == base**(2*x)
     assert ((x*y)**3).extract_multiplicatively(x**2 * y) == x*y**2
     assert ((x*y)**3).extract_multiplicatively(x**4 * y) is None
     assert (2*x).extract_multiplicatively(2) == x
@@ -1282,6 +1323,7 @@ def test_extractions():
     assert (2*x + 3).extract_additively(2*x) == 3
     assert x.extract_additively(0) == x
     assert S(2).extract_additively(x) is None
+    assert S(2.).extract_additively(2.) is S.Zero
     assert S(2.).extract_additively(2) is S.Zero
     assert S(2*x + 3).extract_additively(x + 1) == x + 2
     assert S(2*x + 3).extract_additively(y + 1) is None
@@ -1300,6 +1342,9 @@ def test_extractions():
         x + y + (x + 1)*(x + y) + 3
     assert ((y + 1)*(x + 2*y + 1) + 3).extract_additively(y + 1) == \
         (x + 2*y)*(y + 1) + 3
+    assert (-x - x*I).extract_additively(-x) == -I*x
+    # extraction does not leave artificats, now
+    assert (4*x*(y + 1) + y).extract_additively(x) == x*(4*y + 3) + y
 
     n = Symbol("n", integer=True)
     assert (Integer(-3)).could_extract_minus_sign() is True
@@ -1309,12 +1354,10 @@ def test_extractions():
         (-x + y).could_extract_minus_sign()
     assert (1 - x - y).could_extract_minus_sign() is True
     assert (1 - x + y).could_extract_minus_sign() is False
-    assert ((-x - x*y)/y).could_extract_minus_sign() is True
-    assert (-(x + x*y)/y).could_extract_minus_sign() is True
+    assert ((-x - x*y)/y).could_extract_minus_sign() is False
     assert ((x + x*y)/(-y)).could_extract_minus_sign() is True
     assert ((x + x*y)/y).could_extract_minus_sign() is False
-    assert (x*(-x - x**3)).could_extract_minus_sign() is True
-    assert ((-x - y)/(x + y)).could_extract_minus_sign() is True
+    assert ((-x - y)/(x + y)).could_extract_minus_sign() is False
 
     class sign_invariant(Function, Expr):
         nargs = 1
@@ -1323,13 +1366,12 @@ def test_extractions():
     foo = sign_invariant(x)
     assert foo == -foo
     assert foo.could_extract_minus_sign() is False
-    # The results of each of these will vary on different machines, e.g.
-    # the first one might be False and the other (then) is true or vice versa,
-    # so both are included.
-    assert ((-x - y)/(x - y)).could_extract_minus_sign() is False or \
-           ((-x - y)/(y - x)).could_extract_minus_sign() is False
     assert (x - y).could_extract_minus_sign() is False
     assert (-x + y).could_extract_minus_sign() is True
+    assert (x - 1).could_extract_minus_sign() is False
+    assert (1 - x).could_extract_minus_sign() is True
+    assert (sqrt(2) - 1).could_extract_minus_sign() is True
+    assert (1 - sqrt(2)).could_extract_minus_sign() is False
     # check that result is canonical
     eq = (3*x + 15*y).extract_multiplicatively(3)
     assert eq.args == eq.func(*eq.args).args
@@ -1372,7 +1414,6 @@ def test_coeff():
     assert (2*(n1 + n2)*n2).coeff(n1 + n2, right=1) == n2
     assert (2*(n1 + n2)*n2).coeff(n1 + n2, right=0) == 2
 
-    f = Function('f')
     assert (2*f(x) + 3*f(x).diff(x)).coeff(f(x)) == 2
 
     expr = z*(x + y)**2
@@ -1412,11 +1453,18 @@ def test_coeff():
     assert (n*m + m*n*m).coeff(n, right=True) == m  # = (1 + m)*n*m
     assert (n*m + m*n).coeff(n) == 0
     assert (n*m + o*m*n).coeff(m*n) == o
-    assert (n*m + o*m*n).coeff(m*n, right=1) == 1
-    assert (n*m + n*m*n).coeff(n*m, right=1) == 1 + n  # = n*m*(n + 1)
+    assert (n*m + o*m*n).coeff(m*n, right=True) == 1
+    assert (n*m + n*m*n).coeff(n*m, right=True) == 1 + n  # = n*m*(n + 1)
 
     assert (x*y).coeff(z, 0) == x*y
 
+    assert (x*n + y*n + z*m).coeff(n) == x + y
+    assert (n*m + n*o + o*l).coeff(n, right=True) == m + o
+    assert (x*n*m*n + y*n*m*o + z*l).coeff(m, right=True) == x*n + y*o
+    assert (x*n*m*n + x*n*m*o + z*l).coeff(m, right=True) == n + o
+    assert (x*n*m*n + x*n*m*o + z*l).coeff(m) == x*n
+
+
 def test_coeff2():
     r, kappa = symbols('r, kappa')
     psi = Function("psi")
@@ -1504,6 +1552,11 @@ def test_as_coefficients_dict():
     assert [(3.0*x*y).as_coefficients_dict()[i] for i in check] == \
         [0, 0, 0, 3.0, 0]
     assert (3.0*x*y).as_coefficients_dict()[3.0*x*y] == 0
+    eq = x*(x + 1)*a + x*b + c/x
+    assert eq.as_coefficients_dict(x) == {x: b, 1/x: c,
+        x*(x + 1): a}
+    assert eq.expand().as_coefficients_dict(x) == {x**2: a, x: a + b, 1/x: c}
+    assert x.as_coefficients_dict() == {x: S.One}
 
 
 def test_args_cnc():
@@ -1563,17 +1616,47 @@ def test_free_symbols():
     assert (meter**x).free_symbols == {x}
 
 
+def test_has_free():
+    assert x.has_free(x)
+    assert not x.has_free(y)
+    assert (x + y).has_free(x)
+    assert (x + y).has_free(*(x, z))
+    assert f(x).has_free(x)
+    assert f(x).has_free(f(x))
+    assert Integral(f(x), (f(x), 1, y)).has_free(y)
+    assert not Integral(f(x), (f(x), 1, y)).has_free(x)
+    assert not Integral(f(x), (f(x), 1, y)).has_free(f(x))
+    # simple extraction
+    assert (x + 1 + y).has_free(x + 1)
+    assert not (x + 2 + y).has_free(x + 1)
+    assert (2 + 3*x*y).has_free(3*x)
+    raises(TypeError, lambda: x.has_free({x, y}))
+    s = FiniteSet(1, 2)
+    assert Piecewise((s, x > 3), (4, True)).has_free(s)
+    assert not Piecewise((1, x > 3), (4, True)).has_free(s)
+    # can't make set of these, but fallback will handle
+    raises(TypeError, lambda: x.has_free(y, []))
+
+
+def test_has_xfree():
+    assert (x + 1).has_xfree({x})
+    assert ((x + 1)**2).has_xfree({x + 1})
+    assert not (x + y + 1).has_xfree({x + 1})
+    raises(TypeError, lambda: x.has_xfree(x))
+    raises(TypeError, lambda: x.has_xfree([x]))
+
+
 def test_issue_5300():
     x = Symbol('x', commutative=False)
     assert x*sqrt(2)/sqrt(6) == x*sqrt(3)/3
 
+
 def test_floordiv():
     from sympy.functions.elementary.integers import floor
     assert x // y == floor(x / y)
 
 
 def test_as_coeff_Mul():
-    assert S.Zero.as_coeff_Mul() == (S.One, S.Zero)
     assert Integer(3).as_coeff_Mul() == (Integer(3), Integer(1))
     assert Rational(3, 4).as_coeff_Mul() == (Rational(3, 4), Integer(1))
     assert Float(5.0).as_coeff_Mul() == (Float(5.0), Integer(1))
@@ -1610,7 +1693,6 @@ def test_as_coeff_Add():
 
 
 def test_expr_sorting():
-    f, g = symbols('f,g', cls=Function)
 
     exprs = [1/x**2, 1/x, sqrt(sqrt(x)), sqrt(x), x, sqrt(x)**3, x**2]
     assert sorted(exprs, key=default_sort_key) == exprs
@@ -1654,7 +1736,6 @@ def test_expr_sorting():
 
 
 def test_as_ordered_factors():
-    f, g = symbols('f,g', cls=Function)
 
     assert x.as_ordered_factors() == [x]
     assert (2*x*x**n*sin(x)*cos(x)).as_ordered_factors() \
@@ -1672,7 +1753,6 @@ def test_as_ordered_factors():
 
 
 def test_as_ordered_terms():
-    f, g = symbols('f,g', cls=Function)
 
     assert x.as_ordered_terms() == [x]
     assert (sin(x)**2*cos(x) + sin(x)*cos(x)**2 + 1).as_ordered_terms() \
@@ -1695,16 +1775,17 @@ def test_as_ordered_terms():
     assert ( 4 - 3*I).as_ordered_terms() == [4, -3*I]
     assert (-4 - 3*I).as_ordered_terms() == [-4, -3*I]
 
-    f = x**2*y**2 + x*y**4 + y + 2
+    e = x**2*y**2 + x*y**4 + y + 2
 
-    assert f.as_ordered_terms(order="lex") == [x**2*y**2, x*y**4, y, 2]
-    assert f.as_ordered_terms(order="grlex") == [x*y**4, x**2*y**2, y, 2]
-    assert f.as_ordered_terms(order="rev-lex") == [2, y, x*y**4, x**2*y**2]
-    assert f.as_ordered_terms(order="rev-grlex") == [2, y, x**2*y**2, x*y**4]
+    assert e.as_ordered_terms(order="lex") == [x**2*y**2, x*y**4, y, 2]
+    assert e.as_ordered_terms(order="grlex") == [x*y**4, x**2*y**2, y, 2]
+    assert e.as_ordered_terms(order="rev-lex") == [2, y, x*y**4, x**2*y**2]
+    assert e.as_ordered_terms(order="rev-grlex") == [2, y, x**2*y**2, x*y**4]
 
     k = symbols('k')
     assert k.as_ordered_terms(data=True) == ([(k, ((1.0, 0.0), (1,), ()))], [k])
 
+
 def test_sort_key_atomic_expr():
     from sympy.physics.units import m, s
     assert sorted([-m, s], key=lambda arg: arg.sort_key()) == [-m, s]
@@ -1760,20 +1841,20 @@ def test_issue_5843():
 
 def test_is_constant():
     from sympy.solvers.solvers import checksol
-    Sum(x, (x, 1, 10)).is_constant() is True
-    Sum(x, (x, 1, n)).is_constant() is False
-    Sum(x, (x, 1, n)).is_constant(y) is True
-    Sum(x, (x, 1, n)).is_constant(n) is False
-    Sum(x, (x, 1, n)).is_constant(x) is True
+    assert Sum(x, (x, 1, 10)).is_constant() is True
+    assert Sum(x, (x, 1, n)).is_constant() is False
+    assert Sum(x, (x, 1, n)).is_constant(y) is True
+    assert Sum(x, (x, 1, n)).is_constant(n) is False
+    assert Sum(x, (x, 1, n)).is_constant(x) is True
     eq = a*cos(x)**2 + a*sin(x)**2 - a
-    eq.is_constant() is True
+    assert eq.is_constant() is True
     assert eq.subs({x: pi, a: 2}) == eq.subs({x: pi, a: 3}) == 0
     assert x.is_constant() is False
     assert x.is_constant(y) is True
+    assert log(x/y).is_constant() is False
 
     assert checksol(x, x, Sum(x, (x, 1, n))) is False
     assert checksol(x, x, Sum(x, (x, 1, n))) is False
-    f = Function('f')
     assert f(1).is_constant
     assert checksol(x, x, f(x)) is False
 
@@ -1815,7 +1896,7 @@ def test_equals():
         2*sqrt(2)*x**(S(3)/2)*(1 + 1/(2*x))**(S(5)/2)/(-6 - 3/x)
     ans = sqrt(2*x + 1)*(6*x**2 + x - 1)/15
     diff = i - ans
-    assert diff.equals(0) is False
+    assert diff.equals(0) is None  # should be False, but previously this was False due to wrong intermediate result
     assert diff.subs(x, Rational(-1, 2)/2) == 7*sqrt(2)/120
     # there are regions for x for which the expression is True, for
     # example, when x < -1/2 or x > 0 the expression is zero
@@ -1854,7 +1935,8 @@ def test_equals():
 
 
 def test_random():
-    from sympy import posify, lucas
+    from sympy.functions.combinatorial.numbers import lucas
+    from sympy.simplify.simplify import posify
     assert posify(x)[0]._random() is not None
     assert lucas(n)._random(2, -2, 0, -1, 1) is None
 
@@ -1863,8 +1945,6 @@ def test_random():
 
 
 def test_round():
-    from sympy.abc import x
-
     assert str(Float('0.1249999').round(2)) == '0.12'
     d20 = 12345678901234567890
     ans = S(d20).round(2)
@@ -1887,12 +1967,12 @@ def test_round():
     assert n.round(-1) == 12340
 
     r = Float(str(n)).round(-4)
-    assert r == 10000
+    assert r == 10000.0
 
     assert n.round(-5) == 0
 
     assert str((pi + sqrt(2)).round(2)) == '4.56'
-    assert (10*(pi + sqrt(2))).round(-1) == 50
+    assert (10*(pi + sqrt(2))).round(-1) == 50.0
     raises(TypeError, lambda: round(x + 2, 2))
     assert str(S(2.3).round(1)) == '2.3'
     # rounding in SymPy (as in Decimal) should be
@@ -1925,14 +2005,23 @@ def test_round():
     assert S.Zero.round() == 0
 
     a = (Add(1, Float('1.' + '9'*27, ''), evaluate=0))
-    assert a.round(10) == Float('3.0000000000', '')
-    assert a.round(25) == Float('3.0000000000000000000000000', '')
-    assert a.round(26) == Float('3.00000000000000000000000000', '')
+    assert a.round(10) == Float('3.000000000000000000000000000', '')
+    assert a.round(25) == Float('3.000000000000000000000000000', '')
+    assert a.round(26) == Float('3.000000000000000000000000000', '')
     assert a.round(27) == Float('2.999999999999999999999999999', '')
     assert a.round(30) == Float('2.999999999999999999999999999', '')
 
+    # XXX: Should round set the precision of the result?
+    #      The previous version of the tests above is this but they only pass
+    #      because Floats with unequal precision compare equal:
+    #
+    # assert a.round(10) == Float('3.0000000000', '')
+    # assert a.round(25) == Float('3.0000000000000000000000000', '')
+    # assert a.round(26) == Float('3.00000000000000000000000000', '')
+    # assert a.round(27) == Float('2.999999999999999999999999999', '')
+    # assert a.round(30) == Float('2.999999999999999999999999999', '')
+
     raises(TypeError, lambda: x.round())
-    f = Function('f')
     raises(TypeError, lambda: f(1).round())
 
     # exact magnitude of 10
@@ -1973,17 +2062,17 @@ def test_round():
 
     # check that types match
     for i in range(2):
-        f = float(i)
+        fi = float(i)
         # 2 args
         assert all(type(round(i, p)) is int for p in (-1, 0, 1))
         assert all(S(i).round(p).is_Integer for p in (-1, 0, 1))
-        assert all(type(round(f, p)) is float for p in (-1, 0, 1))
-        assert all(S(f).round(p).is_Float for p in (-1, 0, 1))
+        assert all(type(round(fi, p)) is float for p in (-1, 0, 1))
+        assert all(S(fi).round(p).is_Float for p in (-1, 0, 1))
         # 1 arg (p is None)
         assert type(round(i)) is int
         assert S(i).round().is_Integer
-        assert type(round(f)) is int
-        assert S(f).round().is_Integer
+        assert type(round(fi)) is int
+        assert S(fi).round().is_Integer
 
 
 def test_held_expression_UnevaluatedExpr():
@@ -2048,7 +2137,7 @@ def test_issue_6325():
         (a + b*t)**2 + (c + t*z)**2))/sqrt((a + b*t)**2 + (c + t*z)**2)
     e = sqrt((a + b*t)**2 + (c + z*t)**2)
     assert diff(e, t, 2) == ans
-    e.diff(t, 2) == ans
+    assert e.diff(t, 2) == ans
     assert diff(e, t, 2, simplify=False) != ans
 
 
@@ -2113,6 +2202,13 @@ def test_ExprBuilder():
     assert eb.build() == x**2
 
 
+def test_issue_22020():
+    from sympy.parsing.sympy_parser import parse_expr
+    x = parse_expr("log((2*V/3-V)/C)/-(R+r)*C")
+    y = parse_expr("log((2*V/3-V)/C)/-(R+r)*2")
+    assert x.equals(y) is False
+
+
 def test_non_string_equality():
     # Expressions should not compare equal to strings
     x = symbols('x')
@@ -2137,5 +2233,29 @@ def test_non_string_equality():
 
 def test_21494():
     from sympy.testing.pytest import warns_deprecated_sympy
+
     with warns_deprecated_sympy():
         assert x.expr_free_symbols == {x}
+
+    with warns_deprecated_sympy():
+        assert Basic().expr_free_symbols == set()
+
+    with warns_deprecated_sympy():
+        assert S(2).expr_free_symbols == {S(2)}
+
+    with warns_deprecated_sympy():
+        assert Indexed("A", x).expr_free_symbols == {Indexed("A", x)}
+
+    with warns_deprecated_sympy():
+        assert Subs(x, x, 0).expr_free_symbols == set()
+
+
+def test_Expr__eq__iterable_handling():
+    assert x != range(3)
+
+
+def test_format():
+    assert '{:1.2f}'.format(S.Zero) == '0.00'
+    assert '{:+3.0f}'.format(S(3)) == ' +3'
+    assert '{:23.20f}'.format(pi) == ' 3.14159265358979323846'
+    assert '{:50.48f}'.format(exp(sin(1))) == '2.319776824715853173956590377503266813254904772376'
