@@ -1,25 +1,30 @@
-from sympy.core import Expr, S, Symbol, oo, pi, sympify
-from sympy.core.compatibility import as_int, ordered
-from sympy.core.symbol import _symbol, Dummy, symbols
+from sympy.core import Expr, S, oo, pi, sympify
+from sympy.core.evalf import N
+from sympy.core.sorting import default_sort_key, ordered
+from sympy.core.symbol import _symbol, Dummy, Symbol
 from sympy.functions.elementary.complexes import sign
 from sympy.functions.elementary.piecewise import Piecewise
 from sympy.functions.elementary.trigonometric import cos, sin, tan
-from sympy.geometry.exceptions import GeometryError
+from .ellipse import Circle
+from .entity import GeometryEntity, GeometrySet
+from .exceptions import GeometryError
+from .line import Line, Segment, Ray
+from .point import Point
 from sympy.logic import And
 from sympy.matrices import Matrix
-from sympy.simplify import simplify
-from sympy.utilities import default_sort_key
+from sympy.simplify.simplify import simplify
+from sympy.solvers.solvers import solve
 from sympy.utilities.iterables import has_dups, has_variety, uniq, rotate_left, least_rotation
-from sympy.utilities.misc import func_name
+from sympy.utilities.misc import as_int, func_name
 
-from .entity import GeometryEntity, GeometrySet
-from .point import Point
-from .ellipse import Circle
-from .line import Line, Segment, Ray
+from mpmath.libmp.libmpf import prec_to_dps
 
 import warnings
 
 
+x, y, T = [Dummy('polygon_dummy', real=True) for i in range(3)]
+
+
 class Polygon(GeometrySet):
     """A two-dimensional polygon.
 
@@ -29,13 +34,12 @@ class Polygon(GeometrySet):
     Parameters
     ==========
 
-    vertices : sequence of Points
+    vertices
+        A sequence of points.
 
-    Optional parameters
-    ==========
-
-    n : If > 0, an n-sided RegularPolygon is created. See below.
-        Default value is 0.
+    n : int, optional
+        If $> 0$, an n-sided RegularPolygon is created.
+        Default value is $0$.
 
     Attributes
     ==========
@@ -93,7 +97,7 @@ class Polygon(GeometrySet):
     >>> Polygon((0, 2), (2, 2), (0, 0), (2, 0)).area
     0
 
-    When the the keyword `n` is used to define the number of sides of the
+    When the keyword `n` is used to define the number of sides of the
     Polygon then a RegularPolygon is created and the other arguments are
     interpreted as center, radius and rotation. The unrotated RegularPolygon
     will always have a vertex at Point(r, 0) where `r` is the radius of the
@@ -113,6 +117,8 @@ class Polygon(GeometrySet):
 
     """
 
+    __slots__ = ()
+
     def __new__(cls, *args, n = 0, **kwargs):
         if n:
             args = list(args)
@@ -390,7 +396,7 @@ class Polygon(GeometrySet):
         Returns
         =======
 
-        I_xx, I_yy, I_xy : number or sympy expression
+        I_xx, I_yy, I_xy : number or SymPy expression
                            I_xx, I_yy are second moment of area of a two dimensional polygon.
                            I_xy is product moment of area of a two dimensional polygon.
 
@@ -409,7 +415,7 @@ class Polygon(GeometrySet):
         References
         ==========
 
-        https://en.wikipedia.org/wiki/Second_moment_of_area
+        .. [1] https://en.wikipedia.org/wiki/Second_moment_of_area
 
         """
 
@@ -456,8 +462,8 @@ class Polygon(GeometrySet):
         References
         ==========
 
-        https://skyciv.com/docs/tutorials/section-tutorials/calculating-the-statical-or-first-moment-of-area-of-beam-sections/?cc=BMD
-        https://mechanicalc.com/reference/cross-sections
+        .. [1] https://skyciv.com/docs/tutorials/section-tutorials/calculating-the-statical-or-first-moment-of-area-of-beam-sections/?cc=BMD
+        .. [2] https://mechanicalc.com/reference/cross-sections
 
         Parameters
         ==========
@@ -469,7 +475,7 @@ class Polygon(GeometrySet):
         Returns
         =======
 
-        Q_x, Q_y: number or sympy expressions
+        Q_x, Q_y: number or SymPy expressions
             Q_x is the first moment of area about the x-axis
             Q_y is the first moment of area about the y-axis
             A negative sign indicates that the section modulus is
@@ -520,11 +526,6 @@ class Polygon(GeometrySet):
         plane perpendicular to the object's central axis (i.e. parallel to
         the cross-section)
 
-        References
-        ==========
-
-        https://en.wikipedia.org/wiki/Polar_moment_of_inertia
-
         Examples
         ========
 
@@ -533,6 +534,12 @@ class Polygon(GeometrySet):
         >>> rectangle = Polygon((0, 0), (a, 0), (a, b), (0, b))
         >>> rectangle.polar_second_moment_of_area()
         a**3*b/12 + a*b**3/12
+
+        References
+        ==========
+
+        .. [1] https://en.wikipedia.org/wiki/Polar_moment_of_inertia
+
         """
         second_moment = self.second_moment_of_area()
         return second_moment[0] + second_moment[1]
@@ -546,11 +553,6 @@ class Polygon(GeometrySet):
         ratio of second moment of area to the distance of the extreme end of
         the polygon from the centroidal axis.
 
-        References
-        ==========
-
-        https://en.wikipedia.org/wiki/Section_modulus
-
         Parameters
         ==========
 
@@ -578,6 +580,12 @@ class Polygon(GeometrySet):
         (a*b**2/6, a**2*b/6)
         >>> rectangle.section_modulus(Point(a/4, b/4))
         (-a*b**2/3, -a**2*b/3)
+
+        References
+        ==========
+
+        .. [1] https://en.wikipedia.org/wiki/Section_modulus
+
         """
         x_c, y_c = self.centroid
         if point is None:
@@ -727,7 +735,7 @@ class Polygon(GeometrySet):
         References
         ==========
 
-        [1] http://paulbourke.net/geometry/polygonmesh/#insidepoly
+        .. [1] http://paulbourke.net/geometry/polygonmesh/#insidepoly
 
         """
         p = Point(p, dim=2)
@@ -835,7 +843,6 @@ class Polygon(GeometrySet):
         return Piecewise(*sides)
 
     def parameter_value(self, other, t):
-        from sympy.solvers.solvers import solve
         if not isinstance(other,GeometryEntity):
             other = Point(other, dim=self.ambient_dimension)
         if not isinstance(other,Point):
@@ -843,7 +850,6 @@ class Polygon(GeometrySet):
         if other.free_symbols:
             raise NotImplementedError('non-numeric coordinates')
         unknown = False
-        T = Dummy('t', real=True)
         p = self.arbitrary_point(T)
         for pt, cond in p.args:
             sol = solve(pt - other, T, dict=True)
@@ -967,11 +973,6 @@ class Polygon(GeometrySet):
 
         ValueError: When the line does not intersect the polygon
 
-        References
-        ==========
-
-        https://github.com/sympy/sympy/wiki/A-method-to-return-a-cut-section-of-any-polygon-geometry
-
         Examples
         ========
 
@@ -990,6 +991,12 @@ class Polygon(GeometrySet):
         Point2D(10, 15/2)
         >>> lower_segment.centroid
         Point2D(10, 5/2)
+
+        References
+        ==========
+
+        .. [1] https://github.com/sympy/sympy/wiki/A-method-to-return-a-cut-section-of-any-polygon-geometry
+
         """
         intersection_points = self.intersection(line)
         if not intersection_points:
@@ -998,7 +1005,6 @@ class Polygon(GeometrySet):
         points = list(self.vertices)
         points.append(points[0])
 
-        x, y = symbols('x, y', real=True, cls=Dummy)
         eq = line.equation(x, y)
 
         # considering equation of line to be `ax +by + c`
@@ -1020,7 +1026,7 @@ class Polygon(GeometrySet):
             if compare > 0:
                 if not prev:
                     # if previous point lies below the line, the intersection
-                    # point of the polygon egde and the line has to be included
+                    # point of the polygon edge and the line has to be included
                     edge = Line(point, prev_point)
                     new_point = edge.intersection(line)
                     upper_vertices.append(new_point[0])
@@ -1097,7 +1103,7 @@ class Polygon(GeometrySet):
         Examples
         ========
 
-        >>> from sympy.geometry import Point, Polygon
+        >>> from sympy import Point, Polygon
         >>> square = Polygon(Point(0, 0), Point(0, 1), Point(1, 1), Point(1, 0))
         >>> triangle = Polygon(Point(1, 2), Point(2, 2), Point(2, 1))
         >>> square._do_poly_distance(triangle)
@@ -1107,7 +1113,7 @@ class Polygon(GeometrySet):
         ==========================
 
         Method:
-        [1] http://cgm.cs.mcgill.ca/~orm/mind2p.html
+        [1] https://web.archive.org/web/20150509035744/http://cgm.cs.mcgill.ca/~orm/mind2p.html
         Uses rotating calipers:
         [2] https://en.wikipedia.org/wiki/Rotating_calipers
         and antipodal points:
@@ -1130,7 +1136,8 @@ class Polygon(GeometrySet):
                 e2_max_radius = r
         center_dist = Point.distance(e1_center, e2_center)
         if center_dist <= e1_max_radius + e2_max_radius:
-            warnings.warn("Polygons may intersect producing erroneous output")
+            warnings.warn("Polygons may intersect producing erroneous output",
+                          stacklevel=3)
 
         '''
         Find the upper rightmost vertex of e1 and the lowest leftmost vertex of e2
@@ -1286,9 +1293,6 @@ class Polygon(GeometrySet):
         fill_color : str, optional
             Hex string for fill color. Default is "#66cc99".
         """
-
-        from sympy.core.evalf import N
-
         verts = map(N, self.vertices)
         coords = ["{},{}".format(p.x, p.y) for p in verts]
         path = "M {} L {} z".format(coords[0], " L ".join(coords[1:]))
@@ -1458,7 +1462,7 @@ class RegularPolygon(Polygon):
     Examples
     ========
 
-    >>> from sympy.geometry import RegularPolygon, Point
+    >>> from sympy import RegularPolygon, Point
     >>> r = RegularPolygon(Point(0, 0), 5, 3)
     >>> r
     RegularPolygon(Point2D(0, 0), 5, 3, 0)
@@ -1486,6 +1490,12 @@ class RegularPolygon(Polygon):
         obj._rot = rot % (2*S.Pi/n) if rot.is_number else rot
         return obj
 
+    def _eval_evalf(self, prec=15, **options):
+        c, r, n, a = self.args
+        dps = prec_to_dps(prec)
+        c, r, a = [i.evalf(n=dps, **options) for i in (c, r, a)]
+        return self.func(c, r, n, a)
+
     @property
     def args(self):
         """
@@ -1515,7 +1525,7 @@ class RegularPolygon(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import RegularPolygon
+        >>> from sympy import RegularPolygon
         >>> square = RegularPolygon((0, 0), 1, 4)
         >>> square.area
         2
@@ -1536,7 +1546,7 @@ class RegularPolygon(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import RegularPolygon
+        >>> from sympy import RegularPolygon
         >>> from sympy import sqrt
         >>> s = square_in_unit_circle = RegularPolygon((0, 0), 1, 4)
         >>> s.length
@@ -1566,7 +1576,7 @@ class RegularPolygon(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import RegularPolygon, Point
+        >>> from sympy import RegularPolygon, Point
         >>> rp = RegularPolygon(Point(0, 0), 5, 4)
         >>> rp.center
         Point2D(0, 0)
@@ -1583,7 +1593,7 @@ class RegularPolygon(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import RegularPolygon, Point
+        >>> from sympy import RegularPolygon, Point
         >>> rp = RegularPolygon(Point(0, 0), 5, 4)
         >>> rp.circumcenter
         Point2D(0, 0)
@@ -1610,7 +1620,7 @@ class RegularPolygon(Polygon):
         ========
 
         >>> from sympy import Symbol
-        >>> from sympy.geometry import RegularPolygon, Point
+        >>> from sympy import RegularPolygon, Point
         >>> radius = Symbol('r')
         >>> rp = RegularPolygon(Point(0, 0), radius, 4)
         >>> rp.radius
@@ -1628,7 +1638,7 @@ class RegularPolygon(Polygon):
         ========
 
         >>> from sympy import Symbol
-        >>> from sympy.geometry import RegularPolygon, Point
+        >>> from sympy import RegularPolygon, Point
         >>> radius = Symbol('r')
         >>> rp = RegularPolygon(Point(0, 0), radius, 4)
         >>> rp.circumradius
@@ -1650,7 +1660,7 @@ class RegularPolygon(Polygon):
 
         >>> from sympy import pi
         >>> from sympy.abc import a
-        >>> from sympy.geometry import RegularPolygon, Point
+        >>> from sympy import RegularPolygon, Point
         >>> RegularPolygon(Point(0, 0), 3, 4, pi/4).rotation
         pi/4
 
@@ -1684,7 +1694,7 @@ class RegularPolygon(Polygon):
         ========
 
         >>> from sympy import Symbol
-        >>> from sympy.geometry import RegularPolygon, Point
+        >>> from sympy import RegularPolygon, Point
         >>> radius = Symbol('r')
         >>> rp = RegularPolygon(Point(0, 0), radius, 4)
         >>> rp.apothem
@@ -1702,7 +1712,7 @@ class RegularPolygon(Polygon):
         ========
 
         >>> from sympy import Symbol
-        >>> from sympy.geometry import RegularPolygon, Point
+        >>> from sympy import RegularPolygon, Point
         >>> radius = Symbol('r')
         >>> rp = RegularPolygon(Point(0, 0), radius, 4)
         >>> rp.inradius
@@ -1727,7 +1737,7 @@ class RegularPolygon(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import RegularPolygon, Point
+        >>> from sympy import RegularPolygon, Point
         >>> rp = RegularPolygon(Point(0, 0), 4, 8)
         >>> rp.interior_angle
         3*pi/4
@@ -1752,7 +1762,7 @@ class RegularPolygon(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import RegularPolygon, Point
+        >>> from sympy import RegularPolygon, Point
         >>> rp = RegularPolygon(Point(0, 0), 4, 8)
         >>> rp.exterior_angle
         pi/4
@@ -1777,7 +1787,7 @@ class RegularPolygon(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import RegularPolygon, Point
+        >>> from sympy import RegularPolygon, Point
         >>> rp = RegularPolygon(Point(0, 0), 4, 8)
         >>> rp.circumcircle
         Circle(Point2D(0, 0), 4)
@@ -1802,7 +1812,7 @@ class RegularPolygon(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import RegularPolygon, Point
+        >>> from sympy import RegularPolygon, Point
         >>> rp = RegularPolygon(Point(0, 0), 4, 7)
         >>> rp.incircle
         Circle(Point2D(0, 0), 4*cos(pi/7))
@@ -2007,7 +2017,7 @@ class RegularPolygon(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import RegularPolygon, Point
+        >>> from sympy import RegularPolygon, Point
         >>> rp = RegularPolygon(Point(0, 0), 5, 4)
         >>> rp.vertices
         [Point2D(5, 0), Point2D(0, 5), Point2D(-5, 0), Point2D(0, -5)]
@@ -2073,7 +2083,7 @@ class Triangle(Polygon):
     Examples
     ========
 
-    >>> from sympy.geometry import Triangle, Point
+    >>> from sympy import Triangle, Point
     >>> Triangle(Point(0, 0), Point(4, 0), Point(4, 3))
     Triangle(Point2D(0, 0), Point2D(4, 0), Point2D(4, 3))
 
@@ -2150,7 +2160,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Triangle, Point
+        >>> from sympy import Triangle, Point
         >>> t = Triangle(Point(0, 0), Point(4, 0), Point(4, 3))
         >>> t.vertices
         (Point2D(0, 0), Point2D(4, 0), Point2D(4, 3))
@@ -2181,7 +2191,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Triangle, Point
+        >>> from sympy import Triangle, Point
         >>> t1 = Triangle(Point(0, 0), Point(4, 0), Point(4, 3))
         >>> t2 = Triangle(Point(0, 0), Point(-4, 0), Point(-4, -3))
         >>> t1.is_similar(t2)
@@ -2229,7 +2239,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Triangle, Point
+        >>> from sympy import Triangle, Point
         >>> t1 = Triangle(Point(0, 0), Point(4, 0), Point(4, 3))
         >>> t1.is_equilateral()
         False
@@ -2258,7 +2268,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Triangle, Point
+        >>> from sympy import Triangle, Point
         >>> t1 = Triangle(Point(0, 0), Point(4, 0), Point(2, 4))
         >>> t1.is_isosceles()
         True
@@ -2282,7 +2292,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Triangle, Point
+        >>> from sympy import Triangle, Point
         >>> t1 = Triangle(Point(0, 0), Point(4, 0), Point(1, 4))
         >>> t1.is_scalene()
         True
@@ -2307,7 +2317,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Triangle, Point
+        >>> from sympy import Triangle, Point
         >>> t1 = Triangle(Point(0, 0), Point(4, 0), Point(4, 3))
         >>> t1.is_right()
         True
@@ -2341,7 +2351,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Point, Triangle
+        >>> from sympy import Point, Triangle
         >>> p1, p2, p3 = Point(0, 0), Point(1, 0), Point(0, 1)
         >>> t = Triangle(p1, p2, p3)
         >>> t.altitudes[p1]
@@ -2374,7 +2384,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Point, Triangle
+        >>> from sympy import Point, Triangle
         >>> p1, p2, p3 = Point(0, 0), Point(1, 0), Point(0, 1)
         >>> t = Triangle(p1, p2, p3)
         >>> t.orthocenter
@@ -2404,15 +2414,13 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Point, Triangle
+        >>> from sympy import Point, Triangle
         >>> p1, p2, p3 = Point(0, 0), Point(1, 0), Point(0, 1)
         >>> t = Triangle(p1, p2, p3)
         >>> t.circumcenter
         Point2D(1/2, 1/2)
         """
         a, b, c = [x.perpendicular_bisector() for x in self.sides]
-        if not a.intersection(b):
-            print(a,b,a.intersection(b))
         return a.intersection(b)[0]
 
     @property
@@ -2433,7 +2441,7 @@ class Triangle(Polygon):
         ========
 
         >>> from sympy import Symbol
-        >>> from sympy.geometry import Point, Triangle
+        >>> from sympy import Point, Triangle
         >>> a = Symbol('a')
         >>> p1, p2, p3 = Point(0, 0), Point(1, 0), Point(0, a)
         >>> t = Triangle(p1, p2, p3)
@@ -2459,7 +2467,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Point, Triangle
+        >>> from sympy import Point, Triangle
         >>> p1, p2, p3 = Point(0, 0), Point(1, 0), Point(0, 1)
         >>> t = Triangle(p1, p2, p3)
         >>> t.circumcircle
@@ -2489,7 +2497,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Point, Triangle, Segment
+        >>> from sympy import Point, Triangle, Segment
         >>> p1, p2, p3 = Point(0, 0), Point(1, 0), Point(0, 1)
         >>> t = Triangle(p1, p2, p3)
         >>> from sympy import sqrt
@@ -2528,7 +2536,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Point, Triangle
+        >>> from sympy import Point, Triangle
         >>> p1, p2, p3 = Point(0, 0), Point(1, 0), Point(0, 1)
         >>> t = Triangle(p1, p2, p3)
         >>> t.incenter
@@ -2560,7 +2568,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Point, Triangle
+        >>> from sympy import Point, Triangle
         >>> p1, p2, p3 = Point(0, 0), Point(4, 0), Point(0, 3)
         >>> t = Triangle(p1, p2, p3)
         >>> t.inradius
@@ -2589,7 +2597,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Point, Triangle
+        >>> from sympy import Point, Triangle
         >>> p1, p2, p3 = Point(0, 0), Point(2, 0), Point(0, 2)
         >>> t = Triangle(p1, p2, p3)
         >>> t.incircle
@@ -2622,7 +2630,7 @@ class Triangle(Polygon):
         The exradius touches the side of the triangle to which it is keyed, e.g.
         the exradius touching side 2 is:
 
-        >>> from sympy.geometry import Point, Triangle
+        >>> from sympy import Point, Triangle
         >>> p1, p2, p3 = Point(0, 0), Point(6, 0), Point(0, 2)
         >>> t = Triangle(p1, p2, p3)
         >>> t.exradii[t.sides[2]]
@@ -2631,8 +2639,8 @@ class Triangle(Polygon):
         References
         ==========
 
-        [1] http://mathworld.wolfram.com/Exradius.html
-        [2] http://mathworld.wolfram.com/Excircles.html
+        .. [1] https://mathworld.wolfram.com/Exradius.html
+        .. [2] https://mathworld.wolfram.com/Excircles.html
 
         """
 
@@ -2668,7 +2676,7 @@ class Triangle(Polygon):
         excircle is tangent: The center is keyed, e.g. the excenter of a circle touching
         side 0 is:
 
-        >>> from sympy.geometry import Point, Triangle
+        >>> from sympy import Point, Triangle
         >>> p1, p2, p3 = Point(0, 0), Point(6, 0), Point(0, 2)
         >>> t = Triangle(p1, p2, p3)
         >>> t.excenters[t.sides[0]]
@@ -2682,7 +2690,7 @@ class Triangle(Polygon):
         References
         ==========
 
-        .. [1] http://mathworld.wolfram.com/Excircles.html
+        .. [1] https://mathworld.wolfram.com/Excircles.html
 
         """
 
@@ -2734,7 +2742,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Point, Triangle
+        >>> from sympy import Point, Triangle
         >>> p1, p2, p3 = Point(0, 0), Point(1, 0), Point(0, 1)
         >>> t = Triangle(p1, p2, p3)
         >>> t.medians[p1]
@@ -2766,7 +2774,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Point, Triangle
+        >>> from sympy import Point, Triangle
         >>> p1, p2, p3 = Point(0, 0), Point(1, 0), Point(0, 1)
         >>> t = Triangle(p1, p2, p3)
         >>> t.medial
@@ -2799,7 +2807,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Point, Triangle
+        >>> from sympy import Point, Triangle
         >>> p1, p2, p3 = Point(0, 0), Point(1, 0), Point(0, 1)
         >>> t = Triangle(p1, p2, p3)
         >>> t.nine_point_circle
@@ -2823,7 +2831,7 @@ class Triangle(Polygon):
         Examples
         ========
 
-        >>> from sympy.geometry import Point, Triangle
+        >>> from sympy import Point, Triangle
         >>> p1, p2, p3 = Point(0, 0), Point(1, 0), Point(0, 1)
         >>> t = Triangle(p1, p2, p3)
         >>> t.eulerline
