@@ -1,6 +1,5 @@
 """Compatibility interface between dense and sparse polys. """
 
-from __future__ import print_function, division
 
 from sympy.polys.densearith import dup_add_term
 from sympy.polys.densearith import dmp_add_term
@@ -62,6 +61,8 @@ from sympy.polys.densearith import dup_max_norm
 from sympy.polys.densearith import dmp_max_norm
 from sympy.polys.densearith import dup_l1_norm
 from sympy.polys.densearith import dmp_l1_norm
+from sympy.polys.densearith import dup_l2_norm_squared
+from sympy.polys.densearith import dmp_l2_norm_squared
 from sympy.polys.densearith import dup_expand
 from sympy.polys.densearith import dmp_expand
 from sympy.polys.densebasic import dup_LC
@@ -174,6 +175,10 @@ from sympy.polys.factortools import dmp_zz_diophantine
 from sympy.polys.factortools import dmp_zz_wang_hensel_lifting
 from sympy.polys.factortools import dmp_zz_wang
 from sympy.polys.factortools import dmp_zz_factor
+from sympy.polys.factortools import dup_qq_i_factor
+from sympy.polys.factortools import dup_zz_i_factor
+from sympy.polys.factortools import dmp_qq_i_factor
+from sympy.polys.factortools import dmp_zz_i_factor
 from sympy.polys.factortools import dup_ext_factor
 from sympy.polys.factortools import dmp_ext_factor
 from sympy.polys.factortools import dup_gf_factor
@@ -222,7 +227,7 @@ from sympy.polys.galoistools import (
 from sympy.utilities import public
 
 @public
-class IPolys(object):
+class IPolys:
     symbols = None
     ngens = None
     domain = None
@@ -408,6 +413,11 @@ class IPolys(object):
     def dmp_l1_norm(self, f):
         return dmp_l1_norm(self.to_dense(f), self.ngens-1, self.domain)
 
+    def dup_l2_norm_squared(self, f):
+        return dup_l2_norm_squared(self.to_dense(f), self.domain)
+    def dmp_l2_norm_squared(self, f):
+        return dmp_l2_norm_squared(self.to_dense(f), self.ngens-1, self.domain)
+
     def dup_expand(self, polys):
         return self.from_dense(dup_expand(list(map(self.to_dense, polys)), self.domain))
     def dmp_expand(self, polys):
@@ -811,6 +821,20 @@ class IPolys(object):
         coeff, factors = dmp_zz_factor(self.to_dense(f), self.ngens-1, self.domain)
         return (coeff, [ (self.from_dense(g), k) for g, k in factors ])
 
+    def dup_qq_i_factor(self, f):
+        coeff, factors = dup_qq_i_factor(self.to_dense(f), self.domain)
+        return (coeff, [ (self.from_dense(g), k) for g, k in factors ])
+    def dmp_qq_i_factor(self, f):
+        coeff, factors = dmp_qq_i_factor(self.to_dense(f), self.ngens-1, self.domain)
+        return (coeff, [ (self.from_dense(g), k) for g, k in factors ])
+
+    def dup_zz_i_factor(self, f):
+        coeff, factors = dup_zz_i_factor(self.to_dense(f), self.domain)
+        return (coeff, [ (self.from_dense(g), k) for g, k in factors ])
+    def dmp_zz_i_factor(self, f):
+        coeff, factors = dmp_zz_i_factor(self.to_dense(f), self.ngens-1, self.domain)
+        return (coeff, [ (self.from_dense(g), k) for g, k in factors ])
+
     def dup_ext_factor(self, f):
         coeff, factors = dup_ext_factor(self.to_dense(f), self.domain)
         return (coeff, [ (self.from_dense(g), k) for g, k in factors ])
