@@ -1,4 +1,3 @@
-
 from sympy.polys.galoistools import (
     gf_crt, gf_crt1, gf_crt2, gf_int,
     gf_degree, gf_strip, gf_trunc, gf_normal,
@@ -34,8 +33,9 @@ from sympy.polys.polyerrors import (
 from sympy.polys import polyconfig as config
 
 from sympy.polys.domains import ZZ
-from sympy import pi, nextprime
-from sympy.utilities.pytest import raises
+from sympy.core.numbers import pi
+from sympy.ntheory.generate import nextprime
+from sympy.testing.pytest import raises
 
 
 def test_gf_crt():
@@ -620,6 +620,13 @@ def test_gf_edf():
     assert gf_edf_shoup(f, 2, 3, ZZ) == g
 
 
+def test_issue_23174():
+    f = ZZ.map([1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
+    g = ZZ.map([[1, 0, 0, 1, 1, 1, 0, 0, 1], [1, 1, 1, 0, 1, 0, 1, 1, 1]])
+
+    assert gf_edf_zassenhaus(f, 8, 2, ZZ) == g
+
+
 def test_gf_factor():
     assert gf_factor([], 11, ZZ) == (0, [])
     assert gf_factor([1], 11, ZZ) == (1, [])
