@@ -1,6 +1,11 @@
-from sympy import Symbol, Contains, S, Interval, FiniteSet, oo, Eq
 from sympy.core.expr import unchanged
-from sympy.utilities.pytest import raises
+from sympy.core.numbers import oo
+from sympy.core.relational import Eq
+from sympy.core.singleton import S
+from sympy.core.symbol import Symbol
+from sympy.sets.contains import Contains
+from sympy.sets.sets import (FiniteSet, Interval)
+from sympy.testing.pytest import raises
 
 def test_contains_basic():
     raises(TypeError, lambda: Contains(S.Integers, 1))
@@ -30,13 +35,16 @@ def test_binary_symbols():
     y = Symbol('y')
     z = Symbol('z')
     assert Contains(x, FiniteSet(y, Eq(z, True))
-        ).binary_symbols == set([y, z])
+        ).binary_symbols == {y, z}
 
 
 def test_as_set():
     x = Symbol('x')
     y = Symbol('y')
-    # Contains is a BooleanFunction whose value depends on an arg's
-    # containment in a Set -- rewriting as a Set is not yet implemented
-    raises(NotImplementedError, lambda:
-           Contains(x, FiniteSet(y)).as_set())
+    assert Contains(x, FiniteSet(y)).as_set() == FiniteSet(y)
+    assert Contains(x, S.Integers).as_set() == S.Integers
+    assert Contains(x, S.Reals).as_set() == S.Reals
+
+def test_type_error():
+    # Pass in a parameter not of type "set"
+    raises(TypeError, lambda: Contains(2, None))
